#!/bin/sh

# Copyright 2016 Gigaset Communications GmbH

#-----------------------------------------

LOG_PREFIX="clean-up-utils"
LOG_FILE="/config/clean-up.log"

# Log message
log()
{
    echo "${LOG_PREFIX}: $1"
    if [ -f /usr/bin/logger ]; then
        /usr/bin/logger -t ${LOG_PREFIX} "$1"
    else
        echo "${LOG_PREFIX}: $1" >> ${LOG_FILE}
    fi
}

# Check if string contains substring
strstr()
{
    local ret=2
    if [ "$#" -ne 2 ]; then
        log "[strstr] Expected 2 arguments in strstr, received %s"
    else
        case "$2" in
            *$1*) ret=0 ;;
            *) ;;
        esac
    fi
    return $ret
}

# Clean directory content
# Function will only remove content of directory
# arg1: file path
# arg2: partition name
clean_dir()
{
    local res=1
    local partition="dummy_error"
    log "[clean_dir] Removing directory $1 (partition $2)"
    if [ "$#" -ne 2 ]; then
        log "[clean_dir] Expected 2 arguments, received %s"
    else
        partition="$(df -P "$1" | tail -1 | cut -d' ' -f 1)"
        strstr "$2" "$partition"
        if [ $? -eq 0 ]; then
            rm -rf $1/*
            res=$?
        else
            log "[clean_dir] Fail: $1 is not located on $2 partition"
        fi
    fi
    return $res
}

# Remove single file
# arg1: file path
# arg2: partition name
rm_file()
{
    local res=1
    local partition="dummy_error"
    log "[clean_dir] Removing file $1  (partition $2)"
    if [ "$#" -ne 2 ]; then
        log "[rm_file] Expected 2 arguments, received %s"
    else
        partition="$(df -P "$1" | tail -1 | cut -d' ' -f 1)"
        strstr "$2" "$partition"
        if [ $? -eq 0 ]; then
            rm -f $1
            res=$?
        else
            log "[rm_file] Fail: $1 is not located on $2 partition"
        fi
    fi
    return $res
}

#-----------------------------------------

# Firmware update related clean-up operations
firmware_update_clean()
{
    local res=0
    local result=0
    log "Cleaning firmware update directory"

    clean_dir /config/tmp ubi0_2
    result=$?
    res=$(($res+result))

    return $res
}
#-----------------------------------------

result=0

log "Clean up utils script"

if [ "$#" -ne 1 ]; then
    log "Improper count of arguments in clean-up-utils.sh script, expected 1 argument"
    exit 1
fi

rm LOG_FILE

# Redirect stderr of executed programs to /dev/null
exec 2>/dev/null

case "$1" in
    "firmware-update")
        firmware_update_clean
        result=$(($result+$?)) ;;
    *) 
        log "Unknown parameter" ;;
esac

log "Clean up utils script return: $result"

exit $result
