<?php

/**
 * IPC_MQTT class is MQTT client wrapper
 *
 * @author Marcin Rabczynski <marcin.rabczynski@gigaset.com>
 * @package Maxwell-webui
 * @created 09.2016
 */
class IPC_MQTT implements IPC_Interface
{
    private static $request;
    private static $message;
    private static $instance = null;
    /*
     * @var Mosquitto\Client $connection
     */
    private $connection;

    public static function getInstance()
    {
        try
        {
            if (self::$instance === null) {
                self::$instance = new IPC();
            }
            return self::$instance;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return 0;
        }
    }

    public function sendMessageToProperty($param, $method = 'getV', $value = null)
    {
        $topicPublish = 'web-ui/prop-service';
        $topicSubscribe = 'prop-service/web-ui';
        $message = array('method' => $method, 'param' => $param, 'field' => PropertyService::getField()->webui);

        if ($value !== null) {

            if ($method === 'getV' || $method === 'getColsL')
            {
                $message['fields'] = $value;
            } else if ($method === 'setV')
            {
                $message['value'] = $value;
            } else if ($method === 'setVList')
            {
                $message['list'] = $value;
            } else if ($method === 'makeBackup')
            {

                $message['allSettings'] = $value;
            }
        }
        $responseMessage = $this->publishWait($topicPublish, $topicSubscribe, $message);

        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    public function sendMessageToAudio($value)
    {
        $topicPublish = 'web-ui/audio';

        return $this->sendCustomMessage($topicPublish, $value);
    }

    public function sendMessageToPhoneUI($value)
    {
        $topicPublish = 'web-ui/phone-ui';

        return $this->sendCustomMessage($topicPublish, json_encode($value));
    }

    /**
     * @param $topicPublish string
     * @param $value string
     * @return bool|object
     */
    public function sendCustomMessage($topicPublish, $value)
    {
        $responseMessage = $this->publish($topicPublish, $value);
        if (isset($responseMessage->result))
        {
            $responseMessage = $responseMessage->result;
        }

        return $responseMessage;
    }

    private function connect()
    {
        try {
            $this->connection = new Mosquitto\Client(null, true);
            $this->connection->connect(IPCHOST, 1883, 60);
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function subscribe($_topic)
    {
        try {

            if(self::$instance === null) {
                self::getInstance();
            }

            $this->connect();
            $this->connection->onMessage(function ($_message) {
                $this->messageHandler($_message);
            });
            $this->connection->subscribe($_topic, 1);
            $this->connection->loopForever(5000);
        }
        catch(Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function publish($_topicPublish, $_message)
    {
        try
        {

            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            self::$request = $_message;
            $this->connection->publish($_topicPublish, self::$request);
            loger::debug($_topicPublish, __FILE__, __LINE__);
            loger::debug(self::$request, __FILE__, __LINE__);

            $this->connection->disconnect();
            return true;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function publishWait($_topicPublish, $_topicSubscribe, $_message)
    {
        try
        {

            if(self::$instance === null)
            {
                self::getInstance();
            }

            $this->connect();
            /*
            * @var Mosquitto\Client $connection
            */
            $this->connection->onMessage(function ($_message)
            {
                $this->messageHandler($_message);
            });
            $this->connection->subscribe($_topicSubscribe, 1);
            self::$request = json_encode($_message);
            $this->connection->publish($_topicPublish, self::$request);
            loger::debug($_topicPublish, __FILE__, __LINE__);
            loger::debug(self::$request, __FILE__, __LINE__);

            for($i=1; $i<50; $i++) // 50x0.1s=5s of timeout
            {
                $this->connection->loop(500);

                if(!empty(self::$message))
                {
                    $jsonRequest = json_decode(self::$request);

                    if (($jsonRequest->param === null && self::$message->status === 'Ok') ||
                        (isset(self::$message->param) && $jsonRequest->param === self::$message->param))
                    {

                        loger::debug(var_export($jsonRequest, true), __FILE__, __LINE__);
                        loger::debug(var_export(self::$message, true), __FILE__, __LINE__);

                        $this->connection->disconnect();
                        return self::$message;
                    }
                }
                usleep(100000);
            }
            loger::debug('Response message not recived', __FILE__, __LINE__);
            $this->connection->disconnect();
            return self::$message;
        }
        catch (Exception $ex)
        {
            $exc = new errorexceptionhandler();
            $exc->getException($ex);

            return false;
        }
    }

    private function messageHandler($_message)
    {
        self::$message = json_decode(trim($_message->payload));

        return $_message->payload;
    }
}
